<?php
error_reporting(E_ALL | E_STRICT);

// Load scrypt
require 'lib/functions.php';
require 'lib/scrypt.php';

define('START_TIME'            , microtime(true));
define('CHECKPOINT_FILE'           , 'chash.pos');

// Hashes needed to complete a "block"
$GLOBALS['block_size']          = 100;
$GLOBALS['none_increment']      = (1 / pow(10, 20));

// Automatic saving interval in seconds
$GLOBALS['flush_file_time']     = 30;

/*
 * How long (in seconds) to try to find a proper hash until the best root hash
 * is taken.
 */
$GLOBALS['restart_search_time'] = 1800;

// Hashes per call
$GLOBALS['hash_cycles'] = 2;

// Total restarts
$GLOBALS['total_restarts'] = 0;

// Initial salt (may be overwritten by loading checkpoint file)
$GLOBALS['salt'] = Scrypt::generateSalt();

// Starting difficulty
$GLOBALS['difficulty'] = 2;

// Found hashes
$GLOBALS['found_hashes'] = array([]);

/**
 * Continued-hashing
 *
 * @author		Roland Haeder <roland@mxchange.org>
 * @copyright	Copyright (c) 2013 by Core Developer Team
 * @license		See LICENSE (public-domain)
 */

// Is the check point there?
if (is_readable(CHECKPOINT_FILE)) {
	// Load it
	loadCheckpointFile();
} else {
	// Create nonce (very small)
	initNonce();
}

/*
 * Calculate "genesis" hashes, please note that these "genesis strings" are now
 * known to the public as you can read them here in source code and therefore I
 * will not use them for the real genesis hashes.
 */
$gensisHashes = array(
	// A famous quote from Deus Ex 2 - Invisible War
	multiplehashString('"Informations must be free." - AI Helios from Deus Ex'),
	// My name + URL of my GNUSocial instance
	multipleHashString('Roland Haeder, https://social.mxchange.org'),
	// A famous quote from Linus Torwalds
	multipleHashString('"Software is like sex. Its better when its free." - Linus Torwalds'),
	// Well ...
	multipleHashString('September 11 is a big lie.'),

	// GNU is not Uni*
	multipleHashString('GNU is Not Uni*.'),
	// WINE is not an emulator
	multipleHashString('WINE Is Not an Emulator.'),
	// FlightGear - Fly free!
	multipleHashString('FlightGear - Fly free!'),
	// Quote from Linus Torwalds
	multipleHashString('Your code is shit. Your argument is shit.'),
);

// Calculate first "block"
$genesisBlock = array(
	hashString($gensisHashes[0] . $gensisHashes[3]),
	hashString($gensisHashes[1] . $gensisHashes[2]),
	hashString($gensisHashes[4] . $gensisHashes[7]),
	hashString($gensisHashes[5] . $gensisHashes[6]),
);

// Calulcate final "genesis" hash
$genesisHash = hashString(
	$genesisBlock[3] .
	$genesisBlock[0] .
	$genesisBlock[2] .
	$genesisBlock[1]
);

// Get all elements to get the last part out
$elements = explode('$', $genesisHash);

// This is also the "genesis" hash and first root hash
$GLOBALS['current_hash'] = $genesisHash;
$GLOBALS['root_hash']    = $genesisHash;
$GLOBALS['sum_genesis']  = sumHash($elements[4]);

// Output results
print ('hashes=' . print_r($gensisHashes, true));

// Total reward + hashes
$GLOBALS['total_reward']   = 0;
$GLOBALS['total_hashes']   = 0;
$GLOBALS['total_found']    = 0;
$GLOBALS['total_blocks']   = 0;
$GLOBALS['found_time']     = microtime(true);

// Output again
print ('nonce=' . $GLOBALS['nonce'] . PHP_EOL);
print ('found=' . count($GLOBALS['found_hashes'][$GLOBALS['total_blocks']]) . PHP_EOL);
print ('salt=' . $GLOBALS['salt'] . PHP_EOL);
print ('difficulty=' . $GLOBALS['difficulty'] . PHP_EOL);

// Start "mining"
while (true) {
	// Init hash-per-block counter and hashrate
	$GLOBALS['hashes_block'] = 0;
	$hashRate = 1;

	// Wait for block_size iterations (= found hashes). This is one block
	$timeBlock   = microtime(true);
	$timeDisplay = $timeBlock;
	$GLOBALS['time_flush'] = $timeBlock;

	// Time waited for a good block again (no iteration)
	$timeBadHashes = 0;

	while (count($GLOBALS['found_hashes'][$GLOBALS['total_blocks']]) <= $GLOBALS['block_size']) {
		// Create hash from modulaHash ("genesis hash") and nonce
		$nonceHash = multipleHashString($GLOBALS['nonce'] . $GLOBALS['current_hash']);

		// Calculate sums
		$sumNonce  = calculateSumFromHash($nonceHash);

		// Init counter
		$GLOBALS['iteration'] = 0;
		$GLOBALS['iteration_second'] = 0;

		// Now start the "mining" ...
		$timeHash = microtime(true);
		while ($sumNonce < $GLOBALS['sum_genesis']) {
			// Calculate new nonce
			calculateNonce();

			// And hash again
			$nonceHash = multipleHashString($GLOBALS['nonce'] . $GLOBALS['current_hash']);

			// Calculate sums
			$sumNonce  = calculateSumFromHash($nonceHash);

			// Time spend in loop
			$testTime = abs(microtime(true) - $timeDisplay);

			// Calculate hashrate/sec
			$hashRate = 1 / $testTime * $GLOBALS['iteration_second'] * $GLOBALS['hash_cycles'];

			// Only every second
			if ($testTime >= 1) {
				// Display hash rate
				print ('hashrate=' . round($hashRate) . ' hashes/sec,iterSecond=' . $GLOBALS['iteration_second'] . ' iterations/sec,difficulty=' . $GLOBALS['difficulty'] . PHP_EOL);

				// Reset timer
				$timeDisplay = microtime(true);
				$GLOBALS['iteration_second']  = 0;
			} // END - if

			// Time spend from last flush
			$testTime = abs(microtime(true) - $GLOBALS['time_flush']);

			// Only once per 10 seconds
			if ($testTime >= $GLOBALS['flush_file_time']) {
				// Flush check-point file
				flushCheckPointFile($GLOBALS['current_hash']);
			} // END - if

			// Time spend from last found block
			$testTime = abs(microtime(true) - $GLOBALS['found_time']);

			// Is the last found time to far away?
			if ($testTime >= $GLOBALS['restart_search_time']) {
				// Count up restart and reduce difficulty, but never below 2
				$GLOBALS['total_restarts']++;
				$GLOBALS['difficulty'] = max(2, ($GLOBALS['difficulty'] / 2));

				// Output message
				print('total_restarts=' . $GLOBALS['total_restarts'] . ' - Restarting ...');

				// Count all root (genesis) hashes
				$rootHashes = [];
				foreach ($GLOBALS['found_hashes'] as $block) {
					// "Walk" through all blocks
					foreach ($block as $hash) {
						if (!isset($hash['root_hash'])) {
							// Bad file
							die('INCONSISTENCY: hash=' . print_r($hash, true));
						} // END - if

						if (isset($rootHashes[$hash['root_hash']])) {
							// Count up
							$rootHashes[$hash['root_hash']]++;
						} else {
							// First entry found
							$rootHashes[$hash['root_hash']] = 1;
						}
					} // END - foreach
				} // END - foreach

				// Find best root hash
				$bestRootHash = '';
				$bestRootCount = 0;
				foreach ($rootHashes as $hash => $count) {
					// Debug message
					//* NOISY-DEBUG: */ print ('hash=' . $hash . ',count=' . $count . ',bestRootHash=' . $bestRootHash . ',bestRootCount=' . $bestRootCount . PHP_EOL);

					// Is a better one found?
					if ($count > $bestRootCount) {
						// Remember it
						$bestRootHash  = $hash;
						$bestRootCount = $count;
					} // END - if
				} // END - foreach

				// Output message
				print ('bestRootHash=' . $bestRootHash . ',bestRootCount=' . $bestRootCount . PHP_EOL);

				// Search for latest best root hash
				foreach ($GLOBALS['found_hashes'] as $block) {
					// "Walk" through whole block and search for first appearance of best root hash
					foreach ($block as $idx => $hash) {
						// Is the root hash there?
						if ($hash['root_hash'] == $bestRootHash) {
							// Set found modula hash as new root and current modula hash
							$GLOBALS['root_hash']   = $hash['nonce_hash'];
							setModulaHash($hash['nonce_hash']);
							print ('idx=' . $idx . ',modulaHash=' . $GLOBALS['root_hash'] . ' - Is now new root hash!' . PHP_EOL);

							// Reset "found time" (when a hash was found)
							$GLOBALS['found_time'] = microtime(true);

							// Re-initialize nonce
							initNonce();

							// Abort search
							break;
						} // END - if
					} // END - for
				} // END - foreach
			} // END - if

			// Next round
			$GLOBALS['iteration']++;
			$GLOBALS['iteration_second']++;
			//print ('nonce=' . $GLOBALS['nonce'] . ',iteration=' . $GLOBALS['iteration'] . PHP_EOL);
			//print ('nonceHash=' . $nonceHash . PHP_EOL);
			//print ('sumNonce=' . $sumNonce . PHP_EOL);
			//print ('sumGenesis=' . $GLOBALS['sum_genesis'] . PHP_EOL);
		} // END - while

		// If the iteration is zero, then no hash is found
		if ($GLOBALS['iteration'] == 0) {
			// Bad hash found
			$timeBadHashes += abs(microtime(true) - $timeHash);

			// And next round
			print('BAD:nonce=' . $GLOBALS['nonce'] . PHP_EOL);

			// Nothing found, so calculate new nonce
			calculateNonce();
			continue;
		} // END - if

		// Add amount of hashes per block (multiple-hash)
		$GLOBALS['hashes_block'] += $GLOBALS['iteration'] * $GLOBALS['hash_cycles'] + $GLOBALS['hash_cycles'];

		// Push found hash
		addFoundHash($nonceHash);
	} // END - while

	// Time taken for one
	$timeBlock = abs(microtime(true) - $timeBlock);

	// Calculate reward
	$reward = abs($timeBlock - $timeBadHashes) / $hashRate * $GLOBALS['hashes_block'] / max(1, $GLOBALS['block_size']) * 1000;
	print('timeBlock=' . $timeBlock . ',timeBadHashes=' . $timeBadHashes . ',hashesPerBlock=' . $GLOBALS['hashes_block'] .',reward=' . $reward . PHP_EOL);

	// Double difficulty
	$GLOBALS['difficulty'] = $GLOBALS['difficulty'] * 2;

	// Block completed
	$GLOBALS['total_hashes'] += $GLOBALS['hashes_block'];
	$GLOBALS['total_blocks']++;
	$GLOBALS['hashes_block'] = 0;

	// Init next block
	$GLOBALS['found_hashes'][$GLOBALS['total_blocks']] = [];

	// Calculate new nonce
	calculateNonce();

	// Add reward to total
	$GLOBALS['total_reward'] += $reward;

	// Calculate average block value
	$blockValue = $GLOBALS['total_reward'] / $GLOBALS['total_blocks'] * $GLOBALS['total_hashes'] / ($GLOBALS['block_size'] * $GLOBALS['total_blocks']);

	// Calculate reward per hour (= 3600 seconds)
	$rewardPerHour = $GLOBALS['total_reward'] / abs(microtime(true) - START_TIME) * 3600;

	print ('totalReward=' . $GLOBALS['total_reward'] . ',blockValue=' . $blockValue . ',rewardPerHour=' . $rewardPerHour . PHP_EOL);
} // END - while
