<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Factory;

// Import framework stuff
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

/**
 * A general (base) factory
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseFactory extends BaseFrameworkSystem {
	/**
	 * Total objects generated
	 */
	private static $total = 0;

	/**
	 * Counter of all objects
	 */
	private static $objectCounters = array();

	/**
	 * Protected constructor
	 *
	 * @param	$fullClassName	Name of the real class (not BaseFactory)
	 * @return	void
	 */
	protected function __construct ($fullClassName) {
		// Call parent constructor
		parent::__construct($fullClassName);
	}

	/**
	 * Count given object
	 *
	 * @param	$fullClassName	Name of the class we shall count
	 */
	protected static final function countObject ($fullClassName) {
		// Count it up in total sum
		self::$total++;

		// Do we have an entry?
		if (!isset(self::$objectCounters[$fullClassName])) {
			// No, then generate one
			self::$objectCounters[$fullClassName] = 0;
		} // END - if

		// Count it up again
		//* NOISY-DEBUG: */ print __METHOD__.': className=' .$fullClassName . PHP_EOL;
		self::$objectCounters[$fullClassName]++;
	}

	/**
	 * Static getter for total object count
	 *
	 * @return	$total	Total amount of generated objects
	 */
	public static final function getTotal () {
		return self::$total;
	}

	/**
	 * Static getter for all object counters
	 *
	 * @return	$objectCounters		An array with all object counters
	 */
	public static final function getAllCounters () {
		return self::$objectCounters;
	}

}
