<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Resolver;

// Import framework stuff
use Org\Mxchange\CoreFramework\Generic\FrameworkInterface;
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

/**
 * A generic resolver class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseResolver extends BaseFrameworkSystem {
	/**
	 * Namespace
	 */
	private $namespace = '';

	/**
	 * Class name
	 */
	private $className = '';

	/**
	 * Prefix for class
	 */
	private $classPrefix = '';

	/**
	 * (Last) resolved instance
	 */
	private $resolvedInstance = NULL;

	// Exception constants
	const EXCEPTION_INVALID_COMMAND    = 0x1d0;
	const EXCEPTION_INVALID_CONTROLLER = 0x1d1;
	const EXCEPTION_INVALID_ACTION     = 0x1d2;
	const EXCEPTION_INVALID_STATE      = 0x1d3;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Real name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Getter for namespace
	 *
	 * @return	$namespace	Namespace to look in
	 */
	public final function getNamespace () {
		return $this->namespace;
	}

	/**
	 * Setter for namespace
	 *
	 * @param	$namespace	Namespace to look in
	 * @return	void
	 */
	protected final function setNamespace ($namespace) {
		$this->namespace = (string) $namespace;
	}

	/**
	 * Getter for class name
	 *
	 * @return	$className	Name of the class
	 */
	public final function getClassName () {
		return $this->className;
	}

	/**
	 * Setter for class name
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected final function setClassName ($className) {
		$this->className = (string) $className;
	}

	/**
	 * "Getter" for class name (useable for an object factory)
	 *
	 * @return	$className	Capitalized class name
	 */
	protected function getCapitalizedClassPrefix () {
		// Get class name
		$className = $this->getClassPrefix();

		// And capitalize it
		$className = self::convertToClassName($className);

		// Return it
		return $className;
	}

	/**
	 * Getter for class prefix
	 *
	 * @return	$classPrefix	Last validated classPrefix
	 */
	protected final function getClassPrefix () {
		return $this->classPrefix;
	}

	/**
	 * Setter for class prefix
	 *
	 * @param	$classPrefix	Last validated classPrefix
	 * @return	void
	 */
	protected final function setClassPrefix ($classPrefix) {
		$this->classPrefix = (string) $classPrefix;
	}

	/**
	 * Getter for (last) resolved instance
	 *
	 * @return	$resolvedInstance	Last validated resolvedInstance
	 */
	protected final function getResolvedInstance () {
		return $this->resolvedInstance;
	}

	/**
	 * Setter for (last) resolved instance
	 *
	 * @param	$resolvedInstance	(Last) validated resolved instance
	 * @return	void
	 */
	protected final function setResolvedInstance (FrameworkInterface $resolvedInstance) {
		$this->resolvedInstance = $resolvedInstance;
	}

}
