<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Stacker;

/**
 * A FiLo Stacker class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FiLoStacker extends BaseStacker implements Stackable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of the class Stacker and prepares it for usage
	 *
	 * @return	$stackInstance	An instance of FiLoStacker
	 */
	public static final function createFiLoStacker () {
		// Get a new instance
		$stackInstance = new FiLoStacker();

		// Init the generic stacker
		$stackInstance->initStack('generic');

		// Return the prepared instance
		return $stackInstance;
	}

	/**
	 * Pushs a value on a named stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @param	$value			Value to push on it
	 * @return	void
	 * @throws	StackerFullException	If the stack is full
	 */
	public function pushNamed ($stackerName, $value) {
		// Call the protected method
		parent::addValue($stackerName, $value);
	}

	/**
	 * 'Pops' a value from a named stacker and returns it's value
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value of the current stack entry
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	public function popNamed ($stackerName) {
		// Get the value
		$value = $this->getNamed($stackerName);

		// Call the protected method
		parent::popLast($stackerName);

		// Return the value
		return $value;
	}

	/**
	 * Get value from named stacker
	 *
	 * @param	$stackerName	Name of the stack
	 * @return	$value			Value of last added value
	 * @throws	NoStackerException	If the named stacker was not found
	 * @throws	EmptyStackerException	If the named stacker is empty
	 */
	public function getNamed ($stackerName) {
		// Call the protected method
		return parent::getLastValue($stackerName);
	}

}
