<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Configuration;

// Import framework stuff
use Org\Mxchange\CoreFramework\Generic\FrameworkException;

/**
 * Thrown when the value variable type is NULL, an array, an object or a
 * resource as these values shall not be set as configuration values. Please use
 * FrameworkArrayObject instead!
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 * @todo		These are invalid argument exceptions
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ConfigValueTypeUnsupportedException extends FrameworkException {
	/**
	 * The super constructor for all exceptions
	 *
	 * @param	$message	Message data array
	 * @param	$code		Error code
	 * @return	void
	 */
	public function __construct (array $messageArray, $code) {
		// Construct message
		$message = sprintf('[%s:%d] Configuration key %s has unspported value type %s.',
			$messageArray[0]->__toString(),
			$this->getLine(),
			$messageArray[1],
			gettype($messageArray[2])
		);

		// Call parent exception constructor
		parent::__construct($message, $code);
	}

}
